#----------------------------------------------------------------------------
# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/network/Comms.pm,v 1.18 2009/08/11 11:34:29 aatre Exp $
#
# Veritas Software Corporation
#
# Package Name: Comms
#
# $Author: aatre $
# $Date: 2009/08/11 11:34:29 $
# $Revision: 1.18 $
#
# Description: Provides Various comms related calls.
#
# Authors:   011787 
#----------------------------------------------------------------------------

#-------------------
# Version calls.
#-------------------
use Version;

Version::TellVersion( "\$Header: /project/isv-agent/ACCLib/lib/network/Comms.pm,v 1.18 2009/08/11 11:34:29 aatre Exp $::ETVHeader");
#-----------------------
# End of version calls.
#-----------------------

#---------------------------
# Declare the package name.
#---------------------------
package Comms;

#-----------------------------------------
# Turn on strict for vars, subs and refs.
#-----------------------------------------
use strict;

#----------------------------
# Include necessary packages.
#----------------------------
use LogInt;
use ag_i18n_inc;
use IO::Socket;

#---------------------------------------------
# Include perl built in modules if nessary.
# For CheckPort's socket check and URL get.
#---------------------------------------------
$::bINET6 = 0;

if ( eval 'require IO::Socket::INET6' ) {
  require IO::Socket::INET6;
  ++ $::bINET6;
}

#------------------------------------------------------
# Define the global variables for unix system commands.
#------------------------------------------------------
$::gsCommandIfconfig  = "/sbin/ifconfig";
$::gsCommandGrep      = "/bin/grep";

#--------------------------------------
# Settings for timeouts (in seconds).
#--------------------------------------
$::giPortConnectTimeoutSeconds = 5;
$::giHttpGetTimeoutSeconds     = 10;


=pod

----------------------------------------------------------------------
Function: boolean CheckPort ( IN String  Hostname,
                              IN Integer Port )

Decription: 
  Checks host is reachable on given port.

Parameters: 
  Hostname: Name of the host machine/IPv4/IPv6 address.
  Port    : Port number on which the connection needs to be checked. 
 
Return Value:
  Boolean value in the set of {0,1} where
    0 false
    1 true
----------------------------------------------------------------------

=cut

sub CheckPort {
  my ( $sHost, 
       $iPort ) = @_;

  my $sMyName = ( caller 0 )[3];

  LogInt::Message ( LogInt::LLTrace,
  10001,
  "Comms",
  "CheckPort",
  "Subroutine <%s> called with:
  Host        [%s]
  Port        [%s]",
  $sMyName, $sHost, $iPort );

  #----------------
  # Validate Port..
  #----------------
  if ( $iPort !~ /^\d+$/ ) {
    LogInt::Message ( LogInt::LLError,
    10002,
    "Comms",
    "CheckPort",
    "Invalid Port:[%s] specified",
    $iPort );
    return 0;
  }

  LogInt::Message ( LogInt::LLTrace,
  10003,
  "Comms",
  "CheckPort",
  "Checking Host:[%s] on Port:[%s], TimeoutSeconds:[%s]",
  $sHost,
  $iPort,
  $::giPortConnectTimeout );

  my $oSock = undef;

  if ( $::bINET6 ) {
    $oSock  =  IO::Socket::INET6 -> new ( PeerAddr => $sHost,
                                          PeerPort => $iPort,
                                          Proto    => 'tcp',
                                          Timeout  => $::giPortConnectTimeoutSeconds );
  }
  else {
    $oSock =   IO::Socket::INET -> new ( PeerAddr => $sHost,
                                         PeerPort => $iPort,
                                         Proto    => 'tcp',
                                         Timeout  => $::giPortConnectTimeoutSeconds );
  }

  if ( $oSock ) {
    #----------------
    # Connected OK.
    #----------------
    LogInt::Message ( LogInt::LLTrace,
    10004,
    "Comms",
    "CheckPort",
    "Checking Host:[%s] on Port:[%s], TimeoutSeconds:[%s] (Connected OK)",
    $sHost,
    $iPort,
    $::giPortConnectTimeout );

    close ( $oSock ); # Ignore error..
    return 1;
  } 
  else {
    #-----------------------
    # Failed to connect.
    #-----------------------
    LogInt::Message ( LogInt::LLTrace,
    10005,
    "Comms",
    "CheckPort",
    "Checking Host:[%s] on Port:[%s], TimeoutSeconds:[%s] (Failed to Connect)",
    $sHost,
    $iPort,
    $::giPortConnectTimeout );

    return 0;
  }

}

=pod

-------------------------------------------------------------------
Function: boolean TestURLFromHostPort( IN String   Hostname,
                                       IN Integer  Port,
                                       IN String   URL )

Description:
  Tests the URL on given host with port number.

Parameters:
  Hostname: Name of the host machine.
  Port    : Port number.
  URL     : URL needs to be checked.

Return Value:
  Boolean value in the set of {0,1} where
    0 false
    1 true
-------------------------------------------------------------------

=cut

sub TestURLFromHostPort {
  my ( $sHost,
       $iPort,
       $sURL ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
  10006,
  "Comms",
  "HostPort",
  "Subroutine <%s> called with:
  Host           [%s]
  Port           [%s]
  URL            [%s]",
  $sMyName, $sHost, $iPort, $sURL );

  LogInt::Message( LogInt::LLTrace,
                   10007,
                   "Comms",
		   "TestURLFromHostPort",
		   "Checking Host:[$sHost] on Port:[$iPort], URL:[$sURL]" );
 
  #-------------------------------------------------------------------------
  # $::giPortConnectTimeoutSeconds is used in the connect call.
  # $::giHttpGetTimeoutSeconds is used in the HTTP GET 
  #         (implimented using SIGALRM).
  #-------------------------------------------------------------------------
  my $oSock = undef;

  if ( $::bINET6 ) {
    $oSock  =  IO::Socket::INET6 -> new ( PeerAddr => $sHost,
                                          PeerPort => $iPort,
                                          Proto => 'tcp',
                                          Timeout => $::giPortConnectTimeoutSeconds );
  }
  else {
    $oSock =   IO::Socket::INET -> new ( PeerAddr => $sHost,
                                         PeerPort => $iPort,
                                         Proto => 'tcp',
                                         Timeout => $::giPortConnectTimeoutSeconds );
  }

  if ( $oSock ) {
    #---------------
    # Connected OK.
    #---------------
    LogInt::Message( LogInt::LLTrace, 
    10008,
    "Comms",
    "TestURLFromHostPort", 
    "Host:[%s] on Port:[%s], URL:[%s], TimeoutSeconds:[%s] (Connected OK)",
    $sHost,
    $iPort,
    $sURL,
    $::giPortConnectTimeoutSeconds );
  } 
  else {
    #---------------------
    # Failed to connect.
    #---------------------
    LogInt::Message( LogInt::LLTrace, 
    10009,
    "Comms",
    "TestURLFromHostPort", 
    "Host:[%s] on Port:[%s], URL:[%s], TimeoutSeconds:[%s] ( Failed to Connect)",
    $sHost,
    $iPort,
    $sURL,
    $::giPortConnectTimeoutSeconds );

    #------------------------------------------
    # Return at this point, no port listening.
    #------------------------------------------
    return 0;
  }

  $oSock->autoflush ( 1 );

  #----------------------------------------
  # New line stuff for HTTP commands (GET).
  #----------------------------------------
  my $EOL   = "\015\012";
  my $BLANK = $EOL x 2;
  print $oSock "GET $sURL HTTP/1.0" . $BLANK;

  #-------------------------------------------------------
  # Set up 'try' block for the SIGALARM so we can timeout 
  # after $::giHttpGetTimeoutSeconds.
  #-------------------------------------------------------

  my @lsWebServerResponse = ("Message Set For Timeout Check");

  eval {
    local $SIG{ALRM} = sub { die "sigalarm jump eval\n" };
    alarm $::giHttpGetTimeoutSeconds;

    LogInt::Message ( LogInt::LLTrace,
    10010,
    "Comms",
    "TestURLFromHostPort", 
    "Just before read socket on Port:[$iPort] on Host:[$sHost]" );
    @lsWebServerResponse = <$oSock>;

    #-----------------------------------------------------
    # Cancel the alarm if successed in getting response.
    #-----------------------------------------------------
    alarm 0; 
    LogInt::Message ( LogInt::LLTrace, 
    10011,
    "Comms",
    "TestURLFromHostPort",
    "Read socket completed" );
  };

  chomp $lsWebServerResponse [0];
  close ( $oSock ); # Ignore error code..

  LogInt::Message ( LogInt::LLTrace,
  10012, 
  "Comms", 
  "TestURLFromHostPort",
  "Web server response is:[$lsWebServerResponse[0]]" );

  my ( $sVersion, $iRet ) = split ( /\s+/, $lsWebServerResponse [0] );

  chomp $iRet; 

  #------------------------------------------------------------------------
  # It should match HTTP/1.1 200 OK or HTTP/1.<n> 200 OK, do simple check.
  # The return value should be in between 0 and 200. Do a simple check.
  #------------------------------------------------------------------------
  if ( $iRet > 0 && $iRet <= 200 ) {
    #------------------
    # Got the page OK.
    #------------------
    LogInt::Message ( LogInt::LLTrace, 
    10013,
    "Comms",
    "TestURLFromHostPort",
    "Host:[$sHost] on Port:[$iPort], URL:[$sURL] (Server Returned OK Code between 0 and 200 )" );

    return 1;
  } 
  else {
    #----------------------------
    # Failed to get the page.
    #----------------------------
    LogInt::Message( LogInt::LLTrace, 
    10014,
    "Comms",
    "TestURLFromHostPort", 
    "Host:[$sHost] on Port:[$iPort], URL:[$sURL] (Server FAILED To Return OK Code between 0 and 200 )" );

    return 0;
  }

}


=pod 

----------------------------------------------------------------------------
Function: boolean CheckIfIPExistsLocally( IN String IP )

Description:
  Checks given IP exists on the local machine.
 
Parameters:
  IP:  IP address to be checked. (Ex., 10.212.92.176).

Return Value:
  boolean value in the set of {0,1} where
    0 false
    1 true
----------------------------------------------------------------------------

=cut

sub CheckIfIPExistsLocally {

  my ( $sIP ) = @_;

  my $sMyName = ( caller 0 ) [3];

  LogInt::Message ( LogInt::LLTrace,
  10015,
  "Comms", 
  "CheckIfIPExistsLocally",
  "Subroutine <%s> called with:
  IP           [%s]",
  $sMyName, $sIP );

  my $iIPMatchCount = `$::gsCommandIfconfig -a | $::gsCommandGrep -c "$sIP"`;
  chomp $iIPMatchCount;

  if ( $iIPMatchCount > 0 ) {
    LogInt::Message ( LogInt::LLTrace, 
    10016, 
    "Comms",
    "CheckIfIPExistsLocally",
    "found IP addr:[$sIP]" );
			 
    return 1;
  } 
  else {
    LogInt::Message ( LogInt::LLTrace, 
    10017,
    "Comms", 
    "CheckIfIPExistsLocally",
    "did not find IP addr:[$sIP]" );

    return 0;
  }
}

=pod

 -------------------------------------------------------------------------
 Function : bool IsIPLocal ( IN String Host );


 Description : Tells if IP corresponding to virtual host is plumbed
               locally.
 Parameters :

 $sHost         Hostname/IPv4 address of virtual host

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub IsIPLocal {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sHost ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  LogInt::Message ( LogInt::LLTrace,
  10018,
  "Comms",
  "IsIPLocal",
  "Subroutine <%s> called with:
  Host           [%s]",
  $sMyName, $sHost );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-----------------------------
  # Initialise local variables..
  #-----------------------------
  my ( $iAddr, $sName, $sStrAddr, $iExitCode, @lsCmd ) = ();

  my $sNetCmd = "/bin/netstat";
  my $sNetArgs = $^O eq 'linux'?'-eni':'-ian';

  #----------------------
  # Resolve his request..
  #----------------------
  if ( $iAddr = inet_aton $sHost ) {
    $sName = gethostbyaddr ( $iAddr, AF_INET );
    $sStrAddr = inet_ntoa $iAddr;

    LogInt::Message( LogInt::LLTrace,
                     10019,
                     "Comms",
                     "IsIPLocal",
                     "<%s> Resolved [%s] to Host [%s] IPv4 Address [%s]",
                     $sMyName, $sHost, $sName, $sStrAddr );
  }
  else {
    LogInt::Message( LogInt::LLError,
                     10020,
                     "Comms",
                     "IsIPLocal",
                     "<%s> Could not resolve Host [%s]",
                     $sMyName, $sHost );
    $iReturnCode = 0;
  }
  
  $iReturnCode || return $iReturnCode; # Return, if failed..

  #--------------------------
  # Get locally plumbed IPs..
  #--------------------------
  @lsCmd = grep { s/\n// || 1 }
           readpipe ( "$sNetCmd $sNetArgs" );
  
  #--------------------
  # Check exit status..
  #--------------------
  if ( $iExitCode = $? >> 8 ) {
    LogInt::Message( LogInt::LLError,
                     10021,
                     "Comms",
                     "IsIPLocal",
                     "<%s> Cmd [%s] with Args [%s] failed with Exit Code [%s]",
                     $sNetCmd, $sNetArgs, $iExitCode );
    $iReturnCode = 0;
  }
  elsif ( grep { /\b$sStrAddr\b/o } @lsCmd ) {
    LogInt::Message( LogInt::LLTrace,
                     10022,
                     "Comms",
                     "IsIPLocal",
                     "<%s> IP Address [%s] is plumbed locally",
                     $sMyName, $sStrAddr );
  }
  else {
    LogInt::Message( LogInt::LLTrace,
                     10023,
                     "Comms",
                     "IsIPLocal",
                     "<%s> IP Address [%s] is not plumbed locally",
                     $sMyName, $sStrAddr );
    $iReturnCode = 0;
  }

  return $iReturnCode;
}

#--------------------------------------------
# End with 1, so perl knows its included ok.
#--------------------------------------------
1; 

