# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#------------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/Hares.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2009/07/27 11:59:51 $
# $Revision: 1.16 $
#
# Provides: package Hares 
#
# Functionality: wrapper to hares Functionality.
#
# Authors: 011787
#------------------------------------------------------------------------------

#-----------------
# version calls.
#-----------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/Hares.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $::ETVHeader");
#----------------------
# end of version calls
#----------------------

#-----------------------------------
# Declare the package/module name.
#-----------------------------------
package Hares;

#------------------------------------------
# Turn on strict for subs, vars and refs.
#------------------------------------------
use strict;

#-------------------------------------
# Include all the packages required.
#-------------------------------------
use LogInt;

#------------------------------------------
# ha command to get resource information.
#------------------------------------------
#----------------------------------------------
# Support VAD/VCSOne/VCS 5.0.. Atre 30 Jan 2008
#----------------------------------------------
my $sHaresCmd = ( $ENV { CLUSTER_HOME } ?
                  $ENV { CLUSTER_HOME } : $ENV { VCS_HOME } ) . "/bin/hares";

#--------------------------------------------------------------------------
#  options marked with '*' are available
#
#  hares -add <res> <type> <group>
#  hares -local <res> <attr>
#  hares -global <res> <attr> <value> ... | <key> ... | {<key> <value>} ...
#  hares -delete <res>
#  hares -link <parent res> <child res>
#  hares -unlink <parent res> <child res>
#  hares -clear <res> [-sys <system>]
# *hares -online <res> -sys <system>
# *hares -offline <res> -sys <system>
#  hares -offprop <res> -sys <system>
# *hares -state [<res> -sys <system>]
# *hares -display [<resource(s)>] [-attribute <attribute(s)>]
#          [-group <group(s)>] [-type <type(s)>] [-sys <system(s)>]
# *hares -dep [<resource(s)>]
#  hares -list [<conditional(s)]
#  hares -value <res> <attr> [<system>]
#  hares -probe <res> -sys <system>
#  hares [-help [-modify|-list]]
#----------------------------------------------------------------------------


=pod

------------------------------------------------------------------------------
Function: Boolean Online( IN String Resource, 
                          IN String System )

Description:
  Online the specified resource on the node.

Parameters:
  Resource : Name of the resource.
  System   : Name of the system.

Return Value:
  Boolean value from the set of {0,1} where
        0 - The command fails.
        1 - The command succeeds.
------------------------------------------------------------------------------

=cut

sub Online {

  my ( $sRes,
       $sSystem ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10440,
                   "Hares",
                   "Online",
                   "Subroutine %s called with: Resource Name [%s], System Name [%s]",
                   $sMyName, $sRes, $sSystem );

  #--------------------------------------------------------
  # either works or doesn't, return code reflects status
  # run command - collect stderr as well as stdout
  #--------------------------------------------------------
  my $sHaresOut = `$sHaresCmd -online $sRes -sys $sSystem 2>&1`;
  chomp $sHaresOut;
  my $iReturnCode = $? >> 8;

  #--------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned string and return
  #--------------------------------------------------------
  if ($iReturnCode != 0) {
    LogInt::Message( LogInt::LLError,
                     10441,
                     "Hares",
                     "Online",
                     "Failed with message:[$sHaresOut]" );
    return 0; #error return
  }

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean Offline( IN String Resource, 
                           IN String System )

Description:
  Offline the specified resource on the node.

Parameters:
  Resource : Name of the resource.
  System   : Name of the system.

Return Value:
  Boolean value from the set of {0,1} where
          0 - The command fails.
          1 - The command succeeds.
------------------------------------------------------------------------------

=cut

sub Offline {

  my ( $sRes,
       $sSystem ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10442,
		   "Hares",
		   "Offline",
                   "Subroutine %s called with: Resource Name [%s], System Name [%s]",
                   $sMyName, $sRes, $sSystem );

  #---------------------------------------------------
  #run command - collect stderr as well as stdout
  #---------------------------------------------------
  my $sHaresOut = `$sHaresCmd -offline $sRes -sys $sSystem 2>&1`;
  chomp $sHaresOut;
  my $iReturnCode = $? >> 8;

  #---------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned string and return
  #---------------------------------------------------------
  if ($iReturnCode != 0) {
    LogInt::Message( LogInt::LLError,
                     10443,
		     "Hares",
		     "Offline",
                     "Failed with message:[$sHaresOut]" );
    return 0; #error return
  }

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean GetStateInfoListGivenResource( IN String Resource,
                                                 OUT ArrayOfStrings StateInfo )

Description:
  Get the status of the resource on all the nodes in the cluster.

Parameters:
  Resource  : Name of the Resource.
  StateInfo : Status/state of the resource on each node of the cluster.
              #Resource      Attribute   System        Value
              Apache1326_ip  State       gps-lin3     OFFLINE
              Etc. will be converted into Apache1326_ip State gps-lin3 OFFLINE

Return Value:
  Boolean value from the ser of {0,1} where
          0 - The command fails.
          1 - The command succeeds.
------------------------------------------------------------------------------

=cut

sub GetStateInfoListGivenResource {

  my ( $sRes,
       $rlsStateInfo ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10444,
		   "Hares",
		   "GetStateInfoListGivenResource",
                   "Subroutine %s called with: Resource Name [%s]",
                   $sMyName, $sRes );

  #-------------------------------------------------
  # run command - collect stderr as well as stdout
  #-------------------------------------------------
  my @lsHaresOut = `$sHaresCmd -state $sRes 2>&1`;
  my $iReturnCode = $? >> 8;

  #------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned list of strings and return
  #------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHaresOut = join('\n', @lsHaresOut);
    chomp $sHaresOut;
    LogInt::Message( LogInt::LLError,
                     10445,
		     "Hares",
		     "GetStateInfoListGivenResource",
                     "Failed with message:[$sHaresOut]" );
    return 0; #error return
  }

  #----------------------------------------------------------
  # assign the output to the passed in referance
  # format is #Resource   Attribute        System    Value
  #----------------------------------------------------------
  for my $sEachHaresOut (@lsHaresOut) {
    if($sEachHaresOut =~ /^#/) {
      #ignore lines starting with '#'
    }
    else {
      chomp $sEachHaresOut;
      $sEachHaresOut =~ s/\s+/ /g; #remove multiple spaces
      LogInt::Message( LogInt::LLTraceTrace,
                       10446,
		       "Hares",
		       "GetStateInfoListGivenResource",
                       "Adding [$sEachHaresOut] to rlsStateInfo" );

      push(@$rlsStateInfo, $sEachHaresOut);
    }
  }

  my $iEntryCount = @$rlsStateInfo;
  LogInt::Message( LogInt::LLTrace, 
                   10447,
		   "Hares",
		   "GetStateInfoListGivenResource",
                   "Found [$iEntryCount] entries" );

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean GetStateInfoGivenResourceAndSystem( IN String Resource,
                                                      IN String System, 
                                                      OUT String State )

Description:
  Get the Resource state on the specified node in the cluster.

Parameters:
  Resource : Name of the resource.
  System   : Name of the system/node.
  State    : State of the resource on the node/system which can 
             be one of OFFLINE, ONLINE etc.

Return Value:
  Boolean value from the set of {0,1} where
         0 - The command fails.
         1 - The command succeeds.
------------------------------------------------------------------------------

=cut

sub GetStateInfoGivenResourceAndSystem {

  my ( $sRes, 
       $sSystem,
       $rsState ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10448,
		   "Hares",
		   "GetStateInfoGivenResourceAndSystem",
                   "Subroutine %s called with: Resource Name [%s], System Name [%s]",
                   $sMyName, $sRes, $sSystem );

  #-------------------------------------------------
  # run command - collect stderr as well as stdout
  #-------------------------------------------------
  my $sHaresOut = `$sHaresCmd -state $sRes -sys $sSystem 2>&1`;
  chomp $sHaresOut;
  my $iReturnCode = $? >> 8;

  #------------------------------------------------------
  # check the return code if not 0 then print an error
  # message using the returned string and return
  #------------------------------------------------------
  if ($iReturnCode != 0) {
    LogInt::Message( LogInt::LLError, 
                     10449,
		     "Hares",
		     "GetStateInfoGivenResourceAndSystem",
                     "Failed with message:[$sHaresOut]" );
    return 0; #error return
  }

  LogInt::Message( LogInt::LLTrace, 
                   10450,
		   "Hares",
		   "GetStateInfoGivenResourceAndSystem",
                   "Called with resource name:[$sRes] on system name:[$sSystem] returning state:[$sHaresOut]");

  #-----------------------------------------------
  # assign the output to the passed in referance
  #-----------------------------------------------
  $$rsState = $sHaresOut;

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean GetDependInfoListGivenResource( IN String Resource,
                                                  OUT ListOfStrings DependInfo )

Description:
  Get all the dependent resources of the resource in the cluster.

Parameters:
  Resource   : Name of the resource.
  DependInfo : List containing the resource dependent information.
                 The output will be in the following format:
                   Apache1326   Apache1326_mnt    Apache1326_dg
                   Apache1326   Apache1326_svr_1 Apache1326_mnt
                   Apache1326   Apache1326_svr_2 Apache1326_mnt

Return Value:
  Boolean value from the set of {0,1} where
          0 - The command fails.
          1 - The command succeeds.
------------------------------------------------------------------------------

=cut

sub GetDependInfoListGivenResource {
	
  my ( $sRes,
       $rlsDependInfo ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10451,
		   "Hares",
		   "GetDependInfoListGivenResource",
                   "Subroutine %s called with: Resource Name [%s]",
                   $sMyName, $sRes );

  #-----------------------------------------------------
  # there is a 'special case' with -dep on resources
  # if the return code is !0 and there is no message
  # then its because there was only one resource in the
  # service group (ie its not an error).
  #-----------------------------------------------------

  #------------------------------------------------
  # run command - collect stderr as well as stdout
  #------------------------------------------------
  my @lsHaresOut = `$sHaresCmd -dep $sRes 2>&1`;
  my $iReturnCode = $? >> 8;

  #-------------------------------------------------------
  # check the return code if not 0 then print an error
  # message using the returned list of strings and return
  #-------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHaresOut = undef;
    if(@lsHaresOut) {
      if(@lsHaresOut > 1) {
        $sHaresOut = join('\n', @lsHaresOut);
      } 
      else {
        $sHaresOut = $lsHaresOut[0];
      }
      chomp $sHaresOut;
    }
    else {
      #--------------------------------------------------------
      # this is the 'special case' of an empty return string
      #--------------------------------------------------------
      LogInt::Message( LogInt::LLTrace, 
                       10452,
       		       "Hares",
		       "GetDependInfoListGivenResource",
                       "Assuming no dependants, return code was [$iReturnCode]" );

      #-------------------------------------------
      # return an empty list, set code to 'good'
      #-------------------------------------------
      @$rlsDependInfo = ();

      return 1;
    }
    LogInt::Message( LogInt::LLError,
                     10453,
       		     "Hares",
		     "GetDependInfoListGivenResource",
                     "Failed with message:[$sHaresOut]" );

    return 0; #error return
  }

  #--------------------------------------------------------------
  # Assign the output to the passed in referance
  # remove the first entry as it just contains the header info
  # eg #Group    Parent       Child
  #--------------------------------------------------------------

  shift @lsHaresOut;
  chomp @lsHaresOut;
  @$rlsDependInfo = @lsHaresOut;

  return 1;
}


=pod

------------------------------------------------------------------------------
Function: Boolean Display( IN String Resources,
                           IN String Attributes,
                           IN String Groups, 
                           IN String Types,
                           IN String Systems,
                           OUT ArrayOfStrings Results )

Description:
  Displays the resources attributes on all the systems/nodes specified.

Parameters:
  Resources  : String containing all the resource names.
  Attributes : String containing all the attributes to be displayed.
  Groups     : String containing all the service group names.
  Types      : String containing all type names.
  Systems    : String containing all the system names.
  Results    : Contains the result of display.

Return Value:
  Boolean value from the set of {0,1} where
         0 - The command execution fails.
         1 - The command execution succeeds.
------------------------------------------------------------------------------

=cut

sub Display {

  my ( $sResources,
       $sAttributes,
       $sGroups,
       $sTypes,
       $sSystems, 
       $rlsResults ) = @_;
       
  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace, 
                   10454,
		   "Hares",
		   "Display",
                   "Subroutine %s called with: Resources [%s], Attributes [%s], Groups [%s], Types [%s], Systems [%s]",
                   $sMyName,$sResources, $sAttributes, $sGroups, $sTypes, $sSystems );

  #------------------------------------------------------------------
  # some, all, or none of the above may be populated
  # results of the command are passed back in the $rlsResults param
  # hares -display [<resource(s)>] [-attribute <attribute(s)>]
  #       [-group <group(s)>] [-type <type(s)>] [-sys <system(s)>]
  #------------------------------------------------------------------

  #-------------------------------------------------------------------------
  # go through the passed in params building up the arg list for hares call
  #-------------------------------------------------------------------------

  #---------------------------------------
  # start with the generic display call
  #---------------------------------------
  my $sHaresDisplayCmd = "$sHaresCmd -display"; 

  if($sResources)  { $sHaresDisplayCmd .= " $sResources"; }
  if($sAttributes) { $sHaresDisplayCmd .= " -attribute $sAttributes"; }
  if($sGroups)     { $sHaresDisplayCmd .= " -group $sGroups"; }
  if($sTypes)      { $sHaresDisplayCmd .= " -type $sTypes"; }
  if($sSystems)    { $sHaresDisplayCmd .= " -sys $sSystems"; }

  LogInt::Message( LogInt::LLTrace,
                   10455,
		   "Hares",
		   "Display",
                   "Going to call [$sHaresDisplayCmd]" );

  #---------------------
  # call the command
  #---------------------
  my @lsHaresOut = `$sHaresDisplayCmd 2>&1`;
  my $iReturnCode = $? >> 8;

  #-----------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned data
  #-----------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHaresOut = undef;
    if(@lsHaresOut) {
      if(@lsHaresOut > 1) {
        $sHaresOut = join('\n', @lsHaresOut);
      } 
      else {
        $sHaresOut = $lsHaresOut[0];
      }
      chomp $sHaresOut;
    }
    else {
      $sHaresOut = "no message returned";
    }
    LogInt::Message( LogInt::LLError,
                     10456,
		     "Hares",
		     "Display",
                     "Failed with message:[$sHaresOut]" );

    return 0; #error return
  }

  #--------------------------------------------------------
  # Assign the output to the passed in referance
  #   #Resource       Attribute       System    Value
  #   smbserv1_nbname Group           global    fsapvol1
  # etc..
  #--------------------------------------------------------
  my $sFirstLine = shift @lsHaresOut;
  while( $sFirstLine =~ /.*V-[0-9]+-[0-9]+-[0-9]+\s+No.*/) {
    LogInt::Message( LogInt::LLTrace,
                     10457,
                     "Hares",
                     "Display",
                     "Invalid Group or Resource name" );

    $sFirstLine = shift @lsHaresOut;
  }

  for my $sEachHaresOut (@lsHaresOut) {
    if($sEachHaresOut =~ /^#/) {
      #ignore lines starting with '#'
    } 
    else {
      chomp $sEachHaresOut;
      $sEachHaresOut =~ s/\s+/ /g; #remove multiple spaces
      LogInt::Message( LogInt::LLTraceTrace,
                       10458,
		       "Hares",
		       "Display",
                       "Adding [$sEachHaresOut] to rlsResults" );
			   
      push(@$rlsResults, $sEachHaresOut);
    }
  }

  my $iEntryCount = @$rlsResults;
  LogInt::Message( LogInt::LLTrace,
                   10459,
		   "Hares",
		   "Display",
                   "Found [$iEntryCount] entries" );

  return 1;
}


1;

