# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#-----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/Hatype.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $
#
# Veritas Software Corporation
#
# $Author: aatre $
# $Date: 2009/07/27 11:59:51 $
# $Revision: 1.16 $
# $Tag: $
#
# Provides: package Hatype.
#
# Functionality: wrapper to hatype Functionality
#
# Authors: 011787
#-----------------------------------------------------------------------------

#-----------------
# version calls
#-----------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/Hatype.pm,v 1.16 2009/07/27 11:59:51 aatre Exp $::ETVHeader");
#-----------------------
# end of version calls
#-----------------------

#-----------------------------------
# Declare the package/module name.
#-----------------------------------
package Hatype;

#------------------------------------------
# Turn on strict for subs, vars and refs.
#------------------------------------------
use strict;

#-------------------------------------
# Include all the modules required.
#-------------------------------------
use LogInt;

#-----------------------------------------
# ha command for get types information.
#-----------------------------------------
#----------------------------------------------
# Support VAD/VCSOne/VCS 5.0.. Atre 30 Jan 2008
#----------------------------------------------
my $sHatypeCmd = ( $ENV { CLUSTER_HOME } ?
                   $ENV { CLUSTER_HOME } : $ENV { VCS_HOME } ) . "/bin/hatype";

#----------------------------------------------------------------
#  options marked with '*' are available
#
#  hatype -add <type>
#  hatype -delete <type>
# *hatype -display [<type(s)>] [-attribute <attribute(s)>]
#  hatype -resources <type>
#  hatype -list [<conditional(s)>]
#  hatype -value <type> <attr>
#  hatype [-help [-modify|-list]]
#----------------------------------------------------------------


=pod

-----------------------------------------------------------------------------
Function: Boolean Display( IN String Types, 
                           IN String Attributes,
                           OUT ArrayOfStrings Results )

Description:
  Gets the attribute values for the given type.

Variables:
  Types      : String containing all the names of the types.
  Attributes : String containing all the attributes that are needs to be displayed.
  Results    : Array containg the output of the hatype -display command.
               Entries will be in the following form:
                #Type                Attribute            Value
                Apache      LogLevel           error
                Etc.
 
Return Value:
  Boolean value from the set of {0,1} where
        0 - The command execution fails.
        1 - The command execution succeeds.
-----------------------------------------------------------------------------

=cut

sub Display {
	
  my ( $sTypes, 
       $sAttributes,
       $rlsResults ) = @_;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10410,
                   "Hatype",
                   "Display",
                   "Subroutine %s called with: Types [%s], Attributes [%s]",
                   $sMyName, $sTypes, $sAttributes );

  #-------------------------------------------------------------------
  # some, all, or none of the above may be populated
  # results of the command are passed back in the $rlsResults param
  #
  # hatype -display [<type(s)>] [-attribute <attribute(s)>]
  #-------------------------------------------------------------------

  #-------------------------------------------------------------------------
  # go through the passed in params building up the arg list for hatype call
  #
  # start with the generic display call
  #-------------------------------------------------------------------------
  my $sHatypeDisplayCmd = "$sHatypeCmd -display";

  if($sTypes)      { $sHatypeDisplayCmd .= " $sTypes"; }
  if($sAttributes) { $sHatypeDisplayCmd .= " -attribute $sAttributes"; }

  LogInt::Message( LogInt::LLTrace,
                   10411,
                   "Hatype",
                   "Display",
                   "Going to call [$sHatypeDisplayCmd]" );

  #-------------------
  # call the command
  #-------------------
  my @lsHatypeOut = `$sHatypeDisplayCmd 2>&1`;
  my $iReturnCode = $? >> 8;

  #------------------------------------------------------
  # check the return code if not 0 then print an error 
  # message using the returned data
  #------------------------------------------------------
  if ($iReturnCode != 0) {
    my $sHatypeOut = undef;
    if(@lsHatypeOut > 1) {
      $sHatypeOut = join('\n', @lsHatypeOut);
    } 
    else {
      $sHatypeOut = $lsHatypeOut[0];
    }
    chomp $sHatypeOut;
    LogInt::Message( LogInt::LLError,
                     10412,
                     "Hatype",
                     "Display",
                     "Failed with message:[$sHatypeOut]" );
    
    return 0; #error return
  }

  #-------------------------------------------------
  # assign the output to the passed in referance
  #
  # Type     Attribute              Value
  # Share    AgentClass             TS
  # Share    AgentFailedOn          
  # Share    AgentPriority          
  # etc..
  #-------------------------------------------------

  for my $sEachHatypeOut (@lsHatypeOut) {
    if($sEachHatypeOut =~ /^#/) {
      #ignore lines starting with '#'
    }
    else {
      chomp $sEachHatypeOut;
      $sEachHatypeOut =~ s/\s+/ /g; #remove multiple spaces
      LogInt::Message( LogInt::LLTraceTrace,
                       10413,
                       "Hatype",
                       "Display",
                       "Adding [$sEachHatypeOut] to rlsResults" );
      
      push(@$rlsResults, $sEachHatypeOut);
    }
  }

  my $iEntryCount = @$rlsResults;
  LogInt::Message( LogInt::LLTrace, 
                   10414,
                   "Hatype",
                   "Display",
                   "Found [$iEntryCount] entries" );

  return 1;
}


1; #so perl knows its included ok

