# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#----------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/VCSagentFW.pm,v 1.56 2009/07/27 11:59:51 aatre Exp $
#
# Veritas Software Corporation
# 
# Package name: VCSagentFW
#
# $Author: aatre $
# $Date: 2009/07/27 11:59:51 $
# $Revision: 1.56 $
#
# Description: Simple interface to the Agent Framework
#
# Authors: 011787
#----------------------------------------------------------------------------

#----------------
# Version calls.
#----------------
use Version;
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/VCSagentFW.pm,v 1.56 2009/07/27 11:59:51 aatre Exp $::ETVHeader");

#---------------------------------------
# Initialize global variables for FFDC
#---------------------------------------
$::giState    = undef;
$::giIState   = undef;
@::gsFFDC     = ();
$::gsLogLevel = "INFO";
$::gsUser     = "root";
$::gsEnvFile  = "";
$::giEPTimeout = undef; # EP Timeout..

#------------------------
# Declare the package.
#------------------------
package VCSagentFW;

#---------------------------------------------------------------------------
# Export the well used functions.
# More on exporting at  http://www.perldoc.com/perl5.6.1/lib/Exporter.html.
#---------------------------------------------------------------------------
require Exporter;
@ISA = qw ( Exporter );
@EXPORT = qw ( SetupEntry SetupLogging logCritical logError logWarn logInfo
               logTrace logTraceTrace entryPointExit 
               $HAATTR $HACLUS $HADECRYPT $HAGRP $HAIFCONFIG $HALOG $HALOGIN
               $HARES $HASTATUS $HASYS $HATYPE $HAUSER $NOTIFIER );

#----------------------------------------------------------------------------
# end of export the well used functions.
#----------------------------------------------------------------------------

use strict;


#-------------------------------------
# Include all the packages required.
#-------------------------------------
use LogInt;    #for logging calls
use ag_i18n_inc;
use Sys;
use VCSDefines;
use POSIX qw(tmpnam);
use Fcntl;
use File::Path qw ( mkpath );


use constant MAXBUFFERSIZE => 4096;

#----------------------------------------
# Define our variable types to overload..
#----------------------------------------
my ( $CLUS_HOME, $sPrefix ) = ();
our ( $HAATTR, $HACLUS, $HADECRYPT, $HAGRP, $HAIFCONFIG,
      $HALOG, $HALOGIN, $HARES, $HASTATUS, $HASYS, $HATYPE,
      $HAUSER, $NOTIFIER );

if ( $ENV { CLUSTER_HOME } ) { # VCS 5.0/VCSOne..
   $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else {
   $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /(...)$/;

#------------------------------
# Set if not exported earlier..
#------------------------------
$HAATTR     ||= "$CLUS_HOME/bin/haattr";
$HACLUS     ||= "$CLUS_HOME/bin/haclus";
$HADECRYPT  ||= "$CLUS_HOME/sbin/${sPrefix}decrypt";
$HAGRP      ||= "$CLUS_HOME/bin/hagrp";
$HAIFCONFIG ||= "$CLUS_HOME/bin/haifconfig";
$HALOG      ||= "$CLUS_HOME/bin/halog";
$HALOGIN    ||= "$CLUS_HOME/bin/halogin";
$HARES      ||= "$CLUS_HOME/bin/hares";
$HASTATUS   ||= "$CLUS_HOME/bin/hastatus";
$HASYS      ||= "$CLUS_HOME/bin/hasys";
$HATYPE     ||= "$CLUS_HOME/bin/hatype";
$HAUSER     ||= "$CLUS_HOME/bin/hauser";
$NOTIFIER   ||= "$CLUS_HOME/bin/notifier";

=pod

-------------------------------------------------------------------------
Function: Boolean SetupEntry ( IN   String         EntryPointName,
                               IN   Intger         CategoryID,
                               OUT  Integer        CleanReason,
                               IN   ArrayOfStrings ARGV )

Description:
  Sets up all the entry point logging related information.

Parameters:
  EntryPointName : Name of the entry point of the agent.
  CategoryID     : Unique message category ID for the agent.
  CleanReason    : Reason with which the clean entry point is called 
                   if it is clean entry point.
  ARGV           : List of arguments to setup.
                   if clean entry point is called
                         ARGV should contains first three values 
                             ResourceName 
                             CleanReason 
                             LogLevel
                   otherwise
                         ARGV should contain first two values as
                             ResourceName 
                             LogLevel
                              

Return Value:
  Boolean value in the set of {0, 1} where
  0 - false
  1 - true
-------------------------------------------------------------------------

=cut

sub SetupEntry {

  my ( $sEntryPointName,
       $iCategoryID,
       $riCleanReason,
       $rlsARGV          ) = @_;

  my $sMyName = (caller 0)[3];


  #--------------------------------
  # Set the correct system locale..
  #--------------------------------
  VCSAG_RESET_LANG () if defined &VCSAG_RESET_LANG;


  LogInt::Message( LogInt::LLTrace,
  10262, 
  "VCSagentFW",
  "SetupEntry",
  "Subroutine <%s> called with:
  Entry Point Name         [%s]
  Category ID              [%s]",
  $sMyName, $sEntryPointName, $iCategoryID );

  #-------------------------------------------------------------
  # Call VCSAG_SET_ENVS to set the environment variable for 
  # resource and Category ID.
  #-------------------------------------------------------------
  my $sResourceName = shift @$rlsARGV;
  VCSAG_SET_ENVS( $sResourceName, $iCategoryID );
 
  $ENV{'VCS_LOG_AGENT_NAME'}    ||= $ENV{'VCSONE_LOG_AGENT_NAME'}    ||= "<undef>";
  $ENV{'VCS_LOG_RESOURCE_NAME'} ||= $ENV{'VCSONE_LOG_RESOURCE_NAME'} ||= "<undef>";
  $ENV{'VCS_LOG_SCRIPT_NAME'}   ||= $ENV{'VCSONE_LOG_SCRIPT_NAME'}   ||= "<undef>";
 
  #------------------------------------------------------------------------
  # if the entry point is 'clean' then the next thing is the clean reason.
  #------------------------------------------------------------------------
  if( $sEntryPointName eq "clean" ) {
    $$riCleanReason          = shift @$rlsARGV;
  }

  my $sLogLevel = shift @$rlsARGV;
  $::gsLogLevel = $sLogLevel;

  #----------------------------------------------------------------------
  # Get State and IState into the global variables giState and giIState
  #----------------------------------------------------------------------
  if ((( @$rlsARGV[0] ne "") && (@$rlsARGV[1] ne "")) && 
       ( !(@$rlsARGV[0] =~ /\D/) && !(@$rlsARGV[1] =~ /\D/ )))
  {
    $::giState  =  @$rlsARGV[0];
    $::giIState =  @$rlsARGV[1];
  }

  #--------------------------------------------------------
  # Call setup logging with the args we have been passed.
  #--------------------------------------------------------
  &SetupLogging( $$riCleanReason,
                 $sLogLevel );

  return 1; #all checks passed ok
}


=pod

---------------------------------------------------------------------------
Function: void SetupLogging ( IN Integer CleanReason,
                              IN String LogLevel )

Description: 
  Set the logging level with the string provided and also logs a message
  with clean reason if the entry point is clean. 

Parameters:
  CleanReason : The clean reason with which clean entry point is called.
                If undef, sets it to FFDC.

  LogLevel    : Logging level for the entry point.

Return Value:
  void : No value is returned to the caller.
---------------------------------------------------------------------------

=cut

sub SetupLogging {

  my ( $iCleanReason, 
       $sLogLevel ) = @_;
  
  my $sMyName = (caller 0)[3];
  $iCleanReason ||= 'FFDC'; # Prevent undefs for FFDC..

  LogInt::Message( LogInt::LLTrace,
  10263,
  "VCSagentFW",
  "SetupLogging",
  "Subroutine <%s> called with:
  Clean Reason      [%s]
  Log Level         [%s]",
  $sMyName, $iCleanReason, $sLogLevel );

  #-----------------------------------------------------------------------
  # check and set sLogIntLevelString in order to change the log level.
  #------------------------------------------------------------------------
  if(LogInt::IsValidLLString( $sLogLevel )) {
    LogInt::SetDebugLevelWithString( $sLogLevel );
  } 
  else {
    #---------------------------------------------------------------------
    # don't increment error count for this - it is dealt with elsewhere.
    #---------------------------------------------------------------------
    LogInt::Message( LogInt::LLError,
                     10264,
                     "VCSagentFW",
                     "setupLogging",
                     "Invalid LogLevel String [%s]", $sLogLevel );
    
    LogInt::Message( LogInt::LLInfo,
                     10265,
                     "VCSagentFW",
                     "setupLogging",
                     "Setting LogLevel to Default LogLevel [INFO]" );
  
    LogInt::SetDebugLevelWithString("INFO");
  }

  #------------------------------------------------------------------
  # if we are called from a clean entry point then print an info 
  # message, else just trace
  #------------------------------------------------------------------
  if( $ENV{'VCS_LOG_SCRIPT_NAME'} eq "clean" ) {
    my $sReason = undef;
    VCSDef_CleanReason::GetString($iCleanReason, \$sReason);
    LogInt::Message( LogInt::LLInfo, 
                     10266, 
                     "VCSagentFW",
                     "SetupLogging",
                     "[%s] Entered by resource instance [%s] with clean reason [%s][%s]", 
                     $ENV{'VCS_LOG_SCRIPT_NAME'},
                     $ENV{'VCS_LOG_RESOURCE_NAME'},
                     $iCleanReason,
                     $sReason );
  } 
  else {
    LogInt::Message( LogInt::LLTrace,
                     10267,
                     "VCSagentFW", 
                     "SetupLogging",
                     "[%s] Entered by resource instance [%s]", 
                     $ENV{'VCS_LOG_SCRIPT_NAME'},
                     $ENV{'VCS_LOG_RESOURCE_NAME'});
  }

  return;
}


=pod

---------------------------------------------------------------------------
Function: Boolean verifyLoggingGlobals( IN String CallingSub,
                                        IN Integer MessageId )

Description: 
  Verifies the global variables are set or not before logging a message.

Parameters:
  CallingSub  : Subroutine name which called the logging subroutine.
  MessageId   : Message ID of the message with which logging is called.

Return Value:
  Returns a value from the set of {0, 1} 
     0 - false
     1 - true
---------------------------------------------------------------------------

=cut

sub verifyLoggingGlobals {

  my ( $sCallingSub,
       $iMessageId ) = @_;

  if((!defined $ENV{'VCS_LOG_AGENT_NAME'}) ||
     (!defined $ENV{'VCS_LOG_RESOURCE_NAME'})   ||
     (!defined $ENV{'VCS_LOG_SCRIPT_NAME'}) ){
    #------------------------------------------------
    # Required agent logging globals were not set up.
    # set to <undef>
    #------------------------------------------------
    $ENV{'VCS_LOG_AGENT_NAME'}    ||= $ENV{'VCSONE_LOG_AGENT_NAME'}    ||= "<undef>";
    $ENV{'VCS_LOG_RESOURCE_NAME'} ||= $ENV{'VCSONE_LOG_RESOURCE_NAME'} ||= "<undef>";
    $ENV{'VCS_LOG_SCRIPT_NAME'}   ||= $ENV{'VCSONE_LOG_SCRIPT_NAME'}   ||= "<undef>";

    LogInt::Message( LogInt::LLError,
    10268, 
    "VCSagentFW",
    $sCallingSub, 
    "Called with message id [%s] without first calling SetupEntry",
    $iMessageId );
    return 0;
  } 
  else {
    #------------------------------------------------------
    # all OK required agent logging globals were set up.
    #------------------------------------------------------
    return 1;
  }
}

=pod

---------------------------------------------------------------------------
Function: void logCritical( IN Integer MessageId,
                            IN String  Message,
                            IN ListOfStrings MessageParams )

Description:
  logging call to log the messages of type Critical.

Parameters:
  MessageId     : Unique message ID with which logging is called.
  Message       : Actual message to be logged.
  MessageParams : Any parameters passed to logging message.

Reuturn Value: 
  void : No value is returned to the caller.                            
---------------------------------------------------------------------------

=cut

sub logCritical {

  my ( $iMessageId,
       $sMessage, 
       @lsMessageParams )   = @_;

  if ( &verifyLoggingGlobals("logCritical",$iMessageId) ) {
    LogInt::Message( LogInt::LLCritical,
                     $iMessageId,
                     "", 
                     "",
                     $sMessage,
                     @lsMessageParams );
  }

  return;
}

=pod

---------------------------------------------------------------------------
Function: void logError ( IN Integer MessageId,
                          IN String  Message,
                          IN ListOfStrings MessageParams )

Description:
  logging call to log the messages of type Error.

Parameters:
  MessageId     : Unique message ID with which logging is called.
  Message       : Actual message to be logged.
  MessageParams : Any parameters passed to logging message.

Reuturn Value: 
  void : No value is returned to the caller.                            
---------------------------------------------------------------------------

=cut

sub logError {

  my ( $iMessageId,
       $sMessage, 
       @lsMessageParams )   = @_;

  if ( &verifyLoggingGlobals("logError",$iMessageId) ) {
    LogInt::Message( LogInt::LLError,
                     $iMessageId,
                     "", 
                     "",
                     $sMessage,
                     @lsMessageParams );
  }

  return;
}

=pod

---------------------------------------------------------------------------
Function: void logWarn ( IN Integer MessageId,
                         IN String  Message,
                         IN ListOfStrings MessageParams )

Description:
  logging call to log the messages of type Warnings.

Parameters:
  MessageId     : Unique message ID with which logging is called.
  Message       : Actual message to be logged.
  MessageParams : Any parameters passed to logging message.

Reuturn Value: 
  void : No value is returned to the caller.                            
---------------------------------------------------------------------------

=cut

sub logWarn {

  my ( $iMessageId,
       $sMessage,
       @lsMessageParams ) = @_;

  if ( &verifyLoggingGlobals("logWarn",$iMessageId) ) {
    LogInt::Message( LogInt::LLWarn,
                     $iMessageId,
                     "",
                     "",
                     $sMessage, 
                     @lsMessageParams );
  }

  return;
}

=pod

---------------------------------------------------------------------------
Function: void logInfo ( IN Integer MessageId,
                         IN String  Message,
                         IN ListOfStrings MessageParams )

Description:
  logging call to log the messages of type Info.

Parameters:
  MessageId     : Unique message ID with which logging is called.
  Message       : Actual message to be logged.
  MessageParams : Any parameters passed to logging message.

Reuturn Value: 
  void : No value is returned to the caller.                            
---------------------------------------------------------------------------

=cut

sub logInfo {

  my ( $iMessageId,
       $sMessage,
       @lsMessageParams ) = @_;

  if ( &verifyLoggingGlobals("logInfo",$iMessageId) ) {
    LogInt::Message( LogInt::LLInfo,
                     $iMessageId,
                     "", 
                     "",
                     $sMessage,
                     @lsMessageParams );
   
  }

  return;
}

=pod

---------------------------------------------------------------------------
Function: void logTrace ( IN Integer MessageId,
                          IN String  Message,
                          IN ListOfStrings MessageParams )

Description:
  logging call to log the messages of type Trace.

Parameters:
  MessageId     : Unique message ID with which logging is called.
  Message       : Actual message to be logged.
  MessageParams : Any parameters passed to logging message.

Reuturn Value: 
  void : No value is returned to the caller.                            
---------------------------------------------------------------------------

=cut

sub logTrace {

  my ( $iMessageId,
       $sMessage,
       @lsMessageParams ) = @_;

  if ( &verifyLoggingGlobals("logTrace",$iMessageId) ) {
    LogInt::Message( LogInt::LLTrace, 
                     $iMessageId,
                     "", 
                     "",
                     $sMessage, 
                     @lsMessageParams );
  }

  return;
}

=pod

---------------------------------------------------------------------------
Function: void logTraceTrace ( IN Integer MessageId,
                               IN String  Message,
                               IN ListOfStrings MessageParams )

Description:
  logging call to log the messages of type TraceTrace.

Parameters:
  MessageId     : Unique message ID with which logging is called.
  Message       : Actual message to be logged.
  MessageParams : Any parameters passed to logging message.

Reuturn Value: 
  void : No value is returned to the caller.                            
---------------------------------------------------------------------------

=cut

sub logTraceTrace {

  my ( $iMessageId,
       $sMessage,
       @lsMessageParams ) = @_;

  if ( &verifyLoggingGlobals("logTraceTrace",$iMessageId) ) {
    LogInt::Message( LogInt::LLTrace, 
                     $iMessageId,
                     "",
                     "",
                     $sMessage, 
                     @lsMessageParams );
  }

  return;
}

=pod

 ---------------------------------------------------------------------------
 Function: bool AmIFaulting ( IN Integer ExitCode )

 Description:

   Tells if we are about to fault.

 Parameters:

  ExitCode : Exit value of the entry point.

 Reuturn Value:

  bool :  boolean value in the set of {0,1} where

  0 => false We are not about to fault (steady state)
  1 => true  We are about to fault
 ---------------------------------------------------------------------------

=cut

sub AmIFaulting {
  my ( $iExitCode ) = @_;

  my $sMyName = ( caller 0 ) [3];

  LogInt::Message ( LogInt::LLTrace,
                    10269,
                    "VCSagentFW",
                    "AmIFaulting",
                    "Called with exit code [%s]",
                    $iExitCode );

  #------------------
  # Set return code..
  #------------------
  my $iReturnCode = 0; # Steady state..
  my %hiFaultState = ( 0 => 'FALSE', 1 => 'TRUE' );

  #----------------------------------
  # Decide if we are about to fault..
  #----------------------------------
  if ( defined $::giState && defined $::giIState ) {
    if ( ( VCSDef_ResourceState::ONLINE == $::giState                            &&
           VCSDef_MonitorExit::OFFLINE == $iExitCode                             &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE != $::giIState           &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_PROPAGATE != $::giIState &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_PATH != $::giIState      &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_REVERSE != $::giIState ) ||
         ( VCSDef_ResourceState::OFFLINE == $::giState                           &&
           VCSDef_MonitorExit::OFFLINE == $iExitCode                             &&
           VCSDef_ResourceIState::WAITING_TO_GO_ONLINE == $::giIState ) ) {
      ++ $iReturnCode; # We are faulting..
    }
  }

  LogInt::Message ( LogInt::LLTrace,
                    10270,
                    "VCSagentFW",
                    "AmIFaulting",
                    "Previous steady state changed: [%s]",
                    $hiFaultState { $iReturnCode } );

  return $iReturnCode;
}

=pod

---------------------------------------------------------------------------
Function: void entryPointExit ( IN Integer ExitCode )

Description:
  Exits from the entry point.

Parameters:
  ExitCode : Exit value of the entry point.

Reuturn Value: 
  void : No value is returned to the caller.                            
---------------------------------------------------------------------------

=cut
  
sub entryPointExit {

  my ( $iExitCode ) = @_;
  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{VCS_LOG_RESOURCE_NAME};
  
  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTraceTrace, 
                   10271, 
                   "VCSagentFW", 
                   "entryPointExit",
                   "Subroutine %s called with: Exit code [%s]",
                   $sMyName, $iExitCode );

  LogInt::Message( LogInt::LLTrace, 
                   10272, 
                   "VCSagentFW", 
                   "entryPointExit",
                   "Exiting entry point [%s] with exit code [%s]", 
                   $ENV{'VCS_LOG_SCRIPT_NAME'},
                   $iExitCode );

  if ( defined $::giState && defined $::giIState )
  {
    if ( $ENV{'VCS_LOG_SCRIPT_NAME'} eq "monitor" )
    {
      #----------------------------------------------------------------------------
      # If the resource was online earlier and not going offline, surely  it is 
      # about to fault. Generate trace dump + log long listing of processes running
      #----------------------------------------------------------------------------
      if ((VCSDef_ResourceState::ONLINE == $::giState                            &&
           VCSDef_MonitorExit::OFFLINE == $iExitCode                             &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE != $::giIState           &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_PROPAGATE != $::giIState &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_PATH != $::giIState      &&
           VCSDef_ResourceIState::WAITING_TO_GO_OFFLINE_REVERSE != $::giIState )
          ||
         (VCSDef_ResourceState::OFFLINE == $::giState                            &&
          VCSDef_MonitorExit::OFFLINE == $iExitCode                              &&
          VCSDef_ResourceIState::WAITING_TO_GO_ONLINE == $::giIState             ))
      {
        my $sCleanHasJustRunFile = "/tmp/$sResourceType.$sResourceName.CleanHasJustrun";
        if ( -f $sCleanHasJustRunFile )
        {
          #-------------------------------------------------------------
          # If this is first monitor after a clean has run. No need to 
          # generate the trace dump.
          #-------------------------------------------------------------
          if ( ! unlink($sCleanHasJustRunFile) )
          {
            LogInt::Message( LogInt::LLError,
                       10273,
                       "VCSagentFW",
                       "entryPointExit",
                       "Problems deleting [%s], OS error was [%s]",
                       $sCleanHasJustRunFile,
                       $! );

          }
        }
        else
        {
          #----------------------
          # Generate trace dump.
          #----------------------
          LogInt::Message( LogInt::LLInfo,
                           10274,
                           "VCSagentFW",
                           "entryPointExit",
                           "Generating FFDC messages for resource [%s]",
                           $ENV{'VCS_LOG_RESOURCE_NAME'});

          messageEngineLog(\@::gsFFDC,LogInt::LLInfo) if ( $::gsLogLevel ne "TRACETRACE" );

          #------------------------------------------------------------
          # Dump user environment after sourcing EnvFile if specified
          #------------------------------------------------------------
          my $bSourceEnvFlag = 0;
          my $sCmd           = -x "/bin/env" ? "/bin/env" : "/usr/bin/env";
          my $sCmdArgs       = "";
          my $iTmpReturnCode = 0;
          my @lsCmdOutput    = ();
          my $bTimedOut      = 0;
          my $iTimeOut       = 15;
          my $sOutFileName   = "";
          my $sFromDir       = "";
          Sys::RunWithEnvCmdWithOutputWithTimeOut( $::gsEnvFile,
                                                   $bSourceEnvFlag,
                                                   $sCmd,
                                                   $sCmdArgs,
                                                   \$iTmpReturnCode,
                                                   $::gsUser,
                                                   \@lsCmdOutput,
                                                   \$bTimedOut,
                                                   $iTimeOut,
                                                   $sOutFileName,
                                                   $sFromDir );
          logOutputFromCommand($sCmd,$iTmpReturnCode,\@lsCmdOutput,LogInt::LLInfo);
        }
      }
    }
    elsif ( $ENV{'VCS_LOG_SCRIPT_NAME'} eq "clean" )
    {
      my $sCleanHasJustRunIndicatorFile = "/tmp/$sResourceType.$sResourceName.CleanHasJustrun";
      if ( open (TMP, ">> $sCleanHasJustRunIndicatorFile "))
      {
        close TMP;
      }
      else
      {
        LogInt::Message( LogInt::LLError,
                         10275,
                         "VCSagentFW",
                         "entryPointExit",
                         "<%s>:Couldn't create file [%s], OS error was [%s]",
                         $sMyName, $sCleanHasJustRunIndicatorFile, $! );
      }
    }
  }

  sleep $iExitCode if ( $iExitCode == VCSDef_MonitorExit::TIMEOUT );

  #--------------------------
  # Call exit with the code..
  #--------------------------
  exit $iExitCode;
}

=pod

---------------------------------------------------------------------------
Function: Boolean logOutputFromCommand ( IN String CommandLine,
                                         IN Integer CommandExitCode,
                                         IN ListOfStrings CommandOutput,
                                         IN Integer LogLevel )

Description:
  Logs the output from a failed command.

Parameters:
  CommandLine     : The command executed.
  CommandExitCode : Exit code from the command execution.
  CommandOutput   : Result of the command execution.
  LogLevel        : Logging level for the command output.

Return Value:
  Boolean value from the set of {0,1} where
          0 - false
          1 - true

Caveats:
  1. Logs output from any system commands to a unique tmp file if the
     output is more than 4096 bytes.
  2. If the command output is less than 4096 bytes it won't log the output
     to unique tmp file, it log only to the engine log.
---------------------------------------------------------------------------

=cut

sub logOutputFromCommand {

  my ( $sCommandLine,
       $iCommandExitCode,
       $rlsCommandOutput,
       $iLogLevel ) = @_ ;

  my $sMyName = (caller 0)[3];

  LogInt::Message( LogInt::LLTrace,
                   10276,
                   "VCSagentFW",
                   "logOutputFromCommand",
                   "Subroutine %s called with: Command [%s], Exit Code [%s]",
                   $sMyName, $sCommandLine, $iCommandExitCode );

  my $iMaxHaLogLength     = MAXBUFFERSIZE; 
  chomp @$rlsCommandOutput;
  my $sTotalCommandOutput = join( "\n", @$rlsCommandOutput );
  $sTotalCommandOutput =~ s/"/\\"/g;
  
  if( !defined $iLogLevel ) {
    LogInt::Message( LogInt::LLTrace,
                     10277,
                     "VCSagentFW",
                     "logOutputFromCommand",
                     "LogLevel is not defined. Defaulting to TRACE" );

    $iLogLevel = LogInt::LLTrace;
  }

  if( length $sTotalCommandOutput  >= $iMaxHaLogLength ) {
    my $sTmpFile = tmpnam();
    my $sUString = (split("/", $sTmpFile))[-1];
    #----------------------------------------------------------
    # Frame the temp file with Resource Name, Entry point name
    # Unique ID returned from tmpnam and PID.
    # /tmp/<ResourceName>.<EP>.<uniqueid>.<pid>.
    #----------------------------------------------------------
    my $sTmpLogFile = "/tmp/$ENV{'VCS_LOG_RESOURCE_NAME'}.$ENV{'VCS_LOG_SCRIPT_NAME'}.$sUString.$$"; 
    open( LOGFH, ">$sTmpLogFile" ) or 
         LogInt::Message( LogInt::LLTrace,
                          10278,
                          "VCSagentFW",
                          "logOutputFromCommand",
                          "Can't open a file to write failed command output: [%s], [%s]",
                          $sTmpLogFile, $! );

    print LOGFH "$sTotalCommandOutput";

    close( LOGFH ) or 
         LogInt::Message( LogInt::LLTrace,
                          10279,
                          "VCSagentFW",
                          "logOutputFromCommand",
                          "Can't close the file opened to write failed command output: [%s], [%s]",
                          $sTmpLogFile, $! );

    LogInt::Message( $iLogLevel,
                     10280,
                     "VCSagentFW",
                     "logOutputFromCommand",
                     "Command line [%s] exited with code [%s] , truncating output to approximately [%s] characters [%s].",
                     $sCommandLine,
                     $iCommandExitCode, 
                     $iMaxHaLogLength, 
                     substr( $sTotalCommandOutput, 0, $iMaxHaLogLength ) );

    LogInt::Message( $iLogLevel,
                     10281,
                     "VCSagentFW",
                     "logOutputFromCommand",
                     "The full output is logged to the file [%s]",
                     $sTmpLogFile );
    return 1;
  }
  else {
    LogInt::Message( $iLogLevel,
                     10282,
                     "VCSagentFW",
                     "logOutputFromCommand",
                     "Command line [%s] produced exit code [%s] and generated output [%s]",
                     $sCommandLine,
                     $iCommandExitCode, 
                     $sTotalCommandOutput );
    return 1;
  }

  return 0;
}


=pod
--------------------------------------------------------------------------------
Function: boolean GetVCSObjectHash(  IN  String ObjectName,
                                     IN  String ObjectInstanceName,
                                     OUT StringRef ObjectAttributeValues )

Version: 1.04
 
Description:
  Populates a hash of all Attribute/Value pairs of the specified VCS Object 
  Instance.  

  Supported VCS Objects are:

   Description   Object Name  Command Option
   ===========   ===========  ==============================
   Agent         haagent      -display <Object Instance Name>
   Cluster       haclus       -display
   ServiceGroup  hagrp        -display <Object Instance Name>
   Resource      hares        -display <Object Instance Name>
   System        hasys        -display <Object Instance Name>
   Type          hatype       -display <Object Instance Name>

Parameters:
  ObjectName:            VCS Object Name
  ObjectInstanceName:    VCS Object Instance Name
  ObjectAttributeValues: Reference to a hash to populate
 
Returns:
  A boolean value in the set of {0,1} where 
    0 false
    1 true

Caveats:
  1.  Handles localized attributes by using "System:Attribute" as the key in 
      the passed back hash.  Therefore, callers to this subroutine should 
      first see if the hash key exists for the Attribute that they want: IE:

        my $sAttribute = "LogLevel";
        if( ! exists $hResourceValues{$sAttribute} )
        {
          #----------------------------
          # Handle localized attributes
          #----------------------------
          my $sMyVCS_SysName = "UNKNOWN";
          if( ! Sys::GetSystemNameAsVcsKnowsIt( \$sMyVCS_SysName ) )
          {
            &logError( 1234567, "Could not get VCS SysName" );
            return FAILED; 
          }
          $sAttribute = "$sMyVCS_SysName:$sAttribute";
        }
        $myLogLevel = $hResValues{$sAttribute};

      Therefore, if the desired Attribute key exists in the passed back hash, 
      this would indicate that the Attribute was global.  However, if that 
      key does not exist, the Attribute must be "localized", and the caller needs 
      to obtain his system name (ref: Sys::GetSystemNameAsVcsKnowsIt) and 
      use "SystemName:AttributeName" as the key to get the value of the desired 
      Attribute for the local system.

  2.  Only supports the VCS ojects and associated 'ha' commands listed above.

  3.  The ha command is run under a 10 second alarm.  It might be useful 
      to make this configurable.

  4.  The current implementation (Version 1.04) requires that the "Value" column 
      be the last column of the output (it does not check to make sure).  
      While this is known to be the case with the supported commands, it 
      should probably be more flexible to protect against possible future 
      extensions to VCS.

  5.  The current implementation (Version 1.04) requires that the first line 
      of the output is the "VCS Header line".  It uses this line to determine 
      the column offset of various fields.

  6.  The current implementation (Version 1.04) does not handle embedded comment 
      lines very well (lines beginning with '#').  Some VCS commands are known 
      to produce output with interspersed lines beginning with '#'.

--------------------------------------------------------------------------------
=cut

sub GetVCSObjectHash {

  #----------------------------------------
  # Set up local vars from passed in ones..
  #----------------------------------------
  my ( $sObjectName, $sObjectInstanceName, $rhObjectAttributeValues ) = @_ ;

  #---------------
  # Announce Entry
  #---------------
  my $sMySubName = ( caller 0 )[3];
  LogInt::Message( LogInt::LLTrace, 
                   10283,
                   "VCSagentFW",
                   "GetVCSObjectHash",
                   "Subroutine [%s] called with ObjectName=[%s], ObjectInstanceName=[%s]",
                   $sMySubName, $sObjectName, $sObjectInstanceName );

  my $sEnvFile        = "";   # no EnvFile required
  my $bSourceUserEnv  = 0;    # Do NOT source the user's login profile file

  #----------------------------------------------
  # Support VAD/VCSOne/VCS 5.0.. Atre 30 Jan 2008
  #----------------------------------------------
  my $sCommand = ( $ENV { CLUSTER_HOME } ?
                   $ENV { CLUSTER_HOME } : $ENV { VCS_HOME } ) . "/bin/$sObjectName";

  my $sCommandArgs    = undef;
  my $iCommandExitCode;       # set by called sub
  my $sUser           = "root";
  my @lsCommandOutput=();     # set by called sub
  my $bTimedOut;              # set by called sub
  my $iTimeout        = 10;   # SWAG at a reasonable maximum time to run the VCS command

  if( defined $sObjectInstanceName ) {
    $sCommandArgs = "-display $sObjectInstanceName";
  }
  else {
    $sCommandArgs = "-display";
  }

  if( Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                   $bSourceUserEnv,
                                                   $sCommand,
                                                   $sCommandArgs,
                                                   \$iCommandExitCode,
                                                   $sUser,
                                                   \@lsCommandOutput,
                                                   \$bTimedOut,
                                                   $iTimeout,
                                                   ) ) {
    if( 0 == $iCommandExitCode ) {
      #---------------------------------------------------------
      # Parse the ouput to populate the passed in hash reference
      #---------------------------------------------------------
      my $iAttributeOffset = 0;
      my $iValueOffset     = 0;
      my $iSystemOffset    = 0;
      my ( $sAttributeName, $sAttributeValue, $sSystem ) = ();

      my $sHeaderLine = $lsCommandOutput[0];
      if( $sHeaderLine =~ /^#/ ) {
         shift @lsCommandOutput;
      }

      $iAttributeOffset = index( $sHeaderLine, "Attribute" );
      $iValueOffset = index( $sHeaderLine, "Value" );
      $iSystemOffset = index( $sHeaderLine, "System" );

      #--------------------------------------------------------
      # deal with the '#' in the first column
      # by using offset of 0 if the returned offset is 1
      #--------------------------------------------------------

      if($iAttributeOffset == 1) {
        $iAttributeOffset = 0;
      }

      if($iSystemOffset == 1) {
        $iSystemOffset = 0;
      }

      foreach my $sOutputLine ( @lsCommandOutput ) {
        if( $sOutputLine =~ /^#/ ) {
          next;
        }
        else {
          chomp $sOutputLine;
          ( $sAttributeName ) = split(/\s/, substr( $sOutputLine, $iAttributeOffset ), 2 );
          if ( length($sOutputLine) < $iValueOffset) {
            $sAttributeValue = undef;
          }
          else
          {
            $sAttributeValue = substr( $sOutputLine, $iValueOffset );
          }
          if( $iSystemOffset >= 0 ) 
          {
            ( $sSystem ) = split(/\s/, substr( $sOutputLine, $iSystemOffset ), 2 );
              if( $sSystem ne "global" ) {
                $sAttributeName = $sSystem . ":" . $sAttributeName;
              }
          }
          LogInt::Message( LogInt::LLTrace, 
                           10284, 
                           "VCSagentFW",
                           "GetVCSObjectHash",
                           "Attribute/Value pair:  [%s] [%s]", 
                           $sAttributeName,
                           $sAttributeValue );
          $$rhObjectAttributeValues{$sAttributeName} = $sAttributeValue;
        }
      }
      return 1;
    }
    else {
      VCSagentFW::logOutputFromCommand( "$sCommand $sCommandArgs",
                                        $iCommandExitCode,
                                        \@lsCommandOutput,
                                        LogInt::LLError );
      return 0;
    }
  }
  else {
    #-----------------------------------------------
    # first check to see if the command timed out...
    #-----------------------------------------------
    if( $bTimedOut ) {
      LogInt::Message( LogInt::LLError,
                       10285,
                       "VCSagentFW",
                       "GetVCSObjectHash",
                       "The command line [%s] did not complete within the allotted amount of time ([%s] seconds)",
                       "$sCommand $sCommandArgs", $iTimeout );
      return 0;
    }
    else {
      #----------------------------------------------------------------------
      # log error and return 0 to indicate internal failure in attempted call
      # to RunWithEnvCmdWithOutputWithTimeOut...
      #----------------------------------------------------------------------
      LogInt::Message( LogInt::LLError,
                       10286,
                       "VCSagentFW",
                       "GetVCSObjectHash",
                       "Errors encountered during call to RunWithEnvCmdWithOutputWithTimeOut" );
      return 0;
    }
  }
}

#-------------------------------------------------------------------
# Log message to the engine log. Chomping length if applicable and 
# logging it in multiple logs
#-------------------------------------------------------------------
sub messageEngineLog
{
  my ($lsMessage, $iLogLevel) = @_;
  my $sTotalCommandOutput = join('',@$lsMessage);

  $sTotalCommandOutput =~ s/"/\\"/g;

  my $iMaxHaLogLength = 3000;

  my $len = length $sTotalCommandOutput;
  if ( $len  > $iMaxHaLogLength )
  {
    #-------------------------------
    # Let us log it in chunks of 3K
    #-------------------------------
    my $sTmpString1 = $sTotalCommandOutput;
    my $sTmpString2 = "";
    my $sTmpString3;
    my $iSkipper;
    my $sTillEOL;
    while ( $len > $iMaxHaLogLength )
    {
      $sTmpString2 = substr ($sTmpString1,0,$iMaxHaLogLength);
      $sTmpString1 = substr ($sTmpString1, (length $sTmpString2) - (length $sTmpString1));
      ($sTillEOL) = split ('\n',$sTmpString1);
      $sTmpString2 =~ s/^\n//;
      my $sLogLine = "$sTmpString2"."$sTillEOL";
      LogInt::Message( $iLogLevel,
                       10287,
                       "VCSagentFW",
                       "messageEngineLog",
                       "[%s]", $sLogLine);
      $iSkipper = (length $sTillEOL) - (length $sTmpString1);
      $sTmpString3 = substr ($sTmpString1,$iSkipper);
      $len = length $sTmpString3;
      $sTmpString1 = $sTmpString3;
      if ( $len <= $iMaxHaLogLength )
      {
        LogInt::Message( $iLogLevel,
                         10288,
                         "VCSagentFW",
                         "messageEngineLog",
                         "[%s]", $sTmpString3);
        last;
      }
      last if ( $iSkipper == 0 );
    }
  }
  else
  {
     LogInt::Message( $iLogLevel,
                      10289,
                      "VCSagentFW",
                      "messageEngineLog",
                      "[%s]", $sTotalCommandOutput);
  }

}


=pod

 -------------------------------------------------------------------------
 Function : bool RunSecondLevelMonitor ( IN Integer SecondLevelMonitor );

 Description : Tells if its the correct cycle to perform second level
               monitoring.
 Parameters :

 $iSecondLevelMonitor   Cycle count to run in-depth monitoring

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub RunSecondLevelMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iSecondLevelMonitor ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  LogInt::Message( LogInt::LLTrace,
                   10290,
                   "VCSagentFW",
                   "RunSecondLevelMonitor",
                   "Subroutine %s called with: SecondLevelMonitor [%s]",
                   $sMyName, $iSecondLevelMonitor );


  #---------------------------
  # Set Return Code to false..
  #---------------------------
  my $iReturnCode = 0; # Do not run SLM..

  #------------------------------------
  # Derive current resource type/name..
  #------------------------------------
  my $sResourceType = ( $0 =~ m@([^/]+)@g ) [-2];
  my $sResourceName   = $ENV { VCS_LOG_RESOURCE_NAME };

  #------------------------
  # Setup local variables..
  #------------------------
  my $sSLMDir = "/tmp/.VRTS$sResourceType/$sResourceName/SLM";
  my $sSLMFile = "$sSLMDir/$sResourceName.slm";
  my $iSLMCntr = undef;

  #--------------------------------------------
  # Validate SecondLevelMonitor trivial cases..
  #--------------------------------------------
  if ( 0 == $iSecondLevelMonitor ||
       1 == $iSecondLevelMonitor ) {
    $iReturnCode = $iSecondLevelMonitor; # Assign..

    LogInt::Message( LogInt::LLTrace,
                     10291,
                     "VCSagentFW",
                     "RunSecondLevelMonitor",
                     "<%s> Got request to remove SLM file [%s]. Returning with [%s]",
                     $sMyName, $sSLMFile, $iSecondLevelMonitor );


    unlink $sSLMFile if ( -e $sSLMFile ); # Remove SLM file..
    return $iReturnCode;
  }

  #----------------------------------
  # Create history folder if needed..
  #----------------------------------
  if ( -d $sSLMDir ) {
      LogInt::Message( LogInt::LLTrace,
                       10292,
                       "VCSagentFW",
                       "RunSecondLevelMonitor",
                       "<%s> SLM folder [%s] exists", $sMyName, $sSLMDir );
  }
  elsif ( ! mkpath ( $sSLMDir, 0, 0700 ) ) {
    LogInt::Message( LogInt::LLWarn,
                     10293,
                     "VCSagentFW",
                     "RunSecondLevelMonitor",
                     "<%s> Failed to create SLM folder [%s]",
                     $sMyName, $sSLMDir );

    return 1; # Run SLM..
  }
  
  #--------------------------
  # Create SLM history file..
  #--------------------------
  if ( sysopen ( SLMFILE, $sSLMFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
      LogInt::Message( LogInt::LLTrace,
                       10294,
                       "VCSagentFW",
                       "RunSecondLevelMonitor",
                       "<%s> Opened SLM file [%s] successfully for read/write",
                       $sMyName, $sSLMFile );
  }
  else {
   LogInt::Message( LogInt::LLWarn,
                    10295,
                    "VCSagentFW",
                    "RunSecondLevelMonitor",
                    "<%s> Failed to open SLM file [%s] for read/write",
                    $sMyName, $sSLMFile );

    return 1; # Run SLM..
  }
  
  #---------------
  # Read in file..
  #---------------
  ( $iSLMCntr ) = grep { s/\n// } <SLMFILE>; 

  #-----------------
  # Corrupted file..
  #-----------------
  if ( ! defined $iSLMCntr || $iSLMCntr < 0 ) {

    $iSLMCntr ||= ''; # Avoid undefs..

    LogInt::Message( LogInt::LLTrace,
                     10296,
                     "VCSagentFW",
                     "RunSecondLevelMonitor",
                     "<%s> SLM file [%s] contains [%s]. Re-setting to [%s]",
                     $sMyName, $sSLMFile, $iSLMCntr, $iSecondLevelMonitor );
    $iSLMCntr = $iSecondLevelMonitor; # Set to default..
  }
  
  #-----------------------------------------
  # Update counter, if less than requested..
  #-----------------------------------------
  if ( $iSecondLevelMonitor > $iSLMCntr ) {
    ++ $iSLMCntr;
  }
  else {
    $iSLMCntr = 1;    # Now's the time..
    $iReturnCode = 1; # Run SLM..
  }
  
  #------------------
  # Update the file..
  #------------------
  seek ( SLMFILE, 0, 0 ); # Rewind before update..

  LogInt::Message( LogInt::LLTrace,
                   10297,
                   "VCSagentFW",
                   "RunSecondLevelMonitor",
                   "<%s> Updating SLM file [%s] with [%s]. Return code is [%s]",
                   $sMyName, $sSLMFile, $iSLMCntr, $iReturnCode );

  print SLMFILE "$iSLMCntr\n"; # Update..
  
  close ( SLMFILE ) ||
  LogInt::Message( LogInt::LLWarn,
                   10298,
                   "VCSagentFW",
                   "RunSecondLevelMonitor",
                   "<%s> Close failed on SLM file [%s] after update. OS Error was [%s]",
                   $sMyName, $sSLMFile, $! );

  return $iReturnCode;
}


=pod

 ------------------------------------------------------------------
 Function: bool RunGetProcState ( IN String HostName )

 Description:

 Tells if we need to execute GetProcState.

 When either of the following conditions is true:

 1. We were earlier online || OR ||
 2. The IP is plumbed locally,

 we should do the process check.

 Parameters:

 $sHostName	HostName/IPv4 Address to be used

 Return Value:

 bool :  boolean value in the set of {0,1} where

 0 => false. We do not need to run GetProcState (OFFLINE node)
 1 => true.  We need to execute GetProcState (ONLINE node)
 -----------------------------------------------------------------

=cut

sub RunGetProcState {
  my ( $sHostName ) = @_;
  my $sMyName = ( caller 0 ) [3];

  LogInt::Message ( LogInt::LLTrace,
  10299,
  "VCSagentFW",
  "RunGetProcState",
  "Subroutine <%s> called with HostName/IPv4 address [%s]",
  $sMyName, $sHostName );

  #------------------
  # Set return code..
  #------------------
  my $iReturnCode = 1; # Execute GetProcState..

  #--------------------------------------------------------------------
  # Check if we are not in steady OFFLINE state or
  # the IP address is not plumbed locally or we in steady 
  #--------------------------------------------------------------------
  unless ( VCSagentFW::AmIFaulting ( VCSDef_MonitorExit::OFFLINE ) ||
           Comms::IsIPLocal ( $sHostName ) ) {

    $iReturnCode = 0; # Do not execute GetProcState..

    LogInt::Message ( LogInt::LLTrace,
    10300,
    "VCSagentFW",
    "RunGetProcState",
    "Skip running GetProcState. Returning [%s]", $iReturnCode )
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetEngineTimeout ( IN Bool UseMargin );

 Description : Calculates engine timeout for entry point
 
 Parameters  :

 $bUseMargin      Flag to indicate if safety margin is requested

                  boolean value in the set of {0,1} where
                  0 No safety margin needed
                  1 Safety margin to be used

 Returns:  Time in secs.
 -------------------------------------------------------------------------

=cut

sub GetEngineTimeout {
  #----------------
  # Get Arguments..
  #----------------
  my ( $bUseMargin ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  LogInt::Message ( LogInt::LLTrace,
  10301,
  "VCSagentFW",
  "GetEngineTimeout",
  "Subroutine <%s> called with UseMargin [%s]",
  $sMyName, $bUseMargin );

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sAttribute, $sCommand, $sCommandArgs,
       $iEngineTimeout, $iExitCode ) = ();

  #----------------------------------
  # Grace margins from VCS timeouts..
  #----------------------------------
  my %hiGraceMargin = ( 'MonitorTimeout', 10,   # Seconds..
                        'OnlineTimeout',  10,
                        'OfflineTimeout', 10,
                        'CleanTimeout',   20 );

  #--------------------------------------------------
  # Attempt to get resource level overridden values..
  #--------------------------------------------------
  my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
  my ( $sEntryPointName, $sResourceType ) = reverse $0 =~ m@([^/]+)@g;
  ( $sAttribute = $sEntryPointName ) =~ s/(.*)/\u$1Timeout/;

  #-----------------------------------
  # Query if it has already been set..
  #-----------------------------------
  if ( defined $::giEPTimeout &&    # Defined..
       $::giEPTimeout =~ /^\d+/ ) { # to an integer..
    $iEngineTimeout = $::giEPTimeout;

    LogInt::Message ( LogInt::LLTrace,
    10302,
    "VCSagentFW",
    "GetEngineTimeout",
    "<%s> Using AEPTimeout [%s] as EP timeout value",
    $sMyName, $iEngineTimeout );
  }
  else {
    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    $sCommand = $HARES; # Get global..
    $sCommandArgs = "-value $sResourceName $sAttribute";
  
    #----------------------------------
    # Fire command and collect output..
    #----------------------------------
    ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs 2>/dev/null" ) )
                      =~ s/\n//;
  
    #--------------------
    # Check exit status..
    #--------------------
    if ( $iExitCode = $? >> 8 ) {
      LogInt::Message ( LogInt::LLTrace,
      10303,
      "VCSagentFW",
      "GetEngineTimeout",
      "<%s> Command [%s] with Args [%s] failed with exitcode [%s].",
      $sMyName, $sCommand, $sCommandArgs, $iExitCode );
    }
    elsif ( ! $iEngineTimeout ) { # Undefined value..
      LogInt::Message ( LogInt::LLTrace,
      10304,
      "VCSagentFW",
      "GetEngineTimeout",
      "<%s> Could not get [%s] from engine",
      $sMyName, $sAttribute );
    }
    else { # Success..
      LogInt::Message ( LogInt::LLTrace,
      10305,
      "VCSagentFW",
      "GetEngineTimeout",
      "<%s> Got [%s] as [%s] sec from engine",
      $sMyName, $sAttribute, $iEngineTimeout );
    }
  
    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    unless ( $iEngineTimeout ) {
      $sCommand = $HATYPE; # Get global..
      ( $sAttribute = $sEntryPointName ) =~ s/(.*)/\u$1Timeout/;
      $sCommandArgs = "-value $sResourceType $sAttribute";
    
      #----------------------------------
      # Fire command and collect output..
      #----------------------------------
      ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs" ) ) =~ s/\n//;
    
      #--------------------
      # Check exit status..
      #--------------------
      if ( $iExitCode = $? >> 8 ) {
        LogInt::Message ( LogInt::LLTrace,
        10306,
        "VCSagentFW",
        "GetEngineTimeout",
        "<%s> Command [%s] with Args [%s] failed with exitcode [%s]",
        $sMyName, $sCommand, $sCommandArgs, $iExitCode );

        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      elsif ( ! $iEngineTimeout ) { # Empty value..
        LogInt::Message ( LogInt::LLTrace,
        10307,
        "VCSagentFW",
        "GetEngineTimeout",
        "<%s> Could not get [%s] from engine",
        $sMyName, $sAttribute );

        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      else { # Success..
        LogInt::Message ( LogInt::LLTrace,
        10308,
        "VCSagentFW",
        "GetEngineTimeout",
        "<%s> Got [%s] as [%s] sec from engine",
        $sMyName, $sAttribute, $iEngineTimeout );
      }
    }
    else {
      LogInt::Message ( LogInt::LLTrace,
      10309,
      "VCSagentFW",
      "GetEngineTimeout",
      "<%s> Got [%s] as [%s] sec from engine",
      $sMyName, $sAttribute, $iEngineTimeout );
    }
  }

  #----------------------------
  # Apply margin if requested.. 
  #----------------------------
  if ( $bUseMargin     &&                                    # Margin requested..
       $iEngineTimeout >= $hiGraceMargin { $sAttribute } ) { # And valid..
    $iEngineTimeout -= $hiGraceMargin { $sAttribute };       # Apply..

    LogInt::Message ( LogInt::LLTrace,
    10310,
    "VCSagentFW",
    "GetEngineTimeout",
    "<%s> Setting up [%s] sec margin. Returning [%s]",
    $sMyName, $hiGraceMargin { $sAttribute }, $iEngineTimeout );
  }

  return ( $iEngineTimeout ); # Secs..
}

1; # So Perl knows its included..
