# $Copyright: Copyright (c) 2010 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#---------------------------------------------------------------------------
# $Header: /project/isv-agent/ACCLib/lib/vcs/Version.pm,v 1.6 2009/07/27 11:59:51 aatre Exp $
#
# Veritas Software Corporation
#
# Package Name: Version
#
# $Author: aatre $
# $Date: 2009/07/27 11:59:51 $
# $Revision: 1.6 $
#
# Description: Module to keep track of versions of Perl modules.
#
# Authors: 011787
#---------------------------------------------------------------------------

#----------------------------------------------------------------------------
# To include 'TellVersion' in a file - place the two following lines 
# at the top of the file.
#
# use Version;
# Version::TellVersion("\$Header: 
#   /project/isv-agent/ACCLib/vcs/Version.pm,v 1.5 2005/03/02 01:48:00 
#   vchappav Exp $::ETVHeader"); 
# 
# CVS will *automatically* update the above with the correct header 
#  information (each time the file is commit'd)
#----------------------------------------------------------------------------

#--------------------------------------------------------------------------
# Define this global so anyone who adds the 'TellVersion' call use it to 
# end the trailing '$' needed for CVS string matching 
# (i.e. to keep Perl + CVS happy)
#--------------------------------------------------------------------------
$::ETVHeader = "\$";

#-----------------------------------
# Declare/Announce the pakage name.
#-----------------------------------
package Version;

use strict;

#--------------------------------------
# Add ourselves to the version tag. :)
#--------------------------------------
Version::TellVersion("\$Header: /project/isv-agent/ACCLib/lib/vcs/Version.pm,v 1.6 2009/07/27 11:59:51 aatre Exp $::ETVHeader");

#-----------------------------------------------------------
# Define a hash to hold all versions of the files/packages.
#-----------------------------------------------------------
my %lsFileVersionHash;


=pod

-----------------------------------------------------------------
Function: void TellVersion( IN String VersionInfo )

Description:
  Add's the string to a filename header string hash. Should be 
  called at the top of all Perl modules and scripts which
  use the framework.

Parameters:
  VersionInfo:  Version string.

Return Value: 
  void - returns no value to the caller.
------------------------------------------------------------------

=cut

sub TellVersion {

  my ( $sVersionInfo ) = @_;

  my $sMyName = (caller 0)[3];

  #-----------------------------------
  # Get the caller filename to index.
  #-----------------------------------
  my @lsCallerInfo = caller;
  my $sFileName = $lsCallerInfo[1];

  #-----------------------------
  # Add versin string to hash.
  #-----------------------------
  $lsFileVersionHash{$sFileName} = $sVersionInfo;
  
  return;
}


=pod

----------------------------------------------------------------
Function: boolean CheckForVersionRequest( IN String FirstArg ) 

Description: 
  Prints all versions of the module if it is requested.

Parameters:
  FirstArg: A string containing "--versions".

Return value:
  Boolean value in the set of {0,1} where
  0 - false
  1 - true
----------------------------------------------------------------

=cut

sub CheckForVersionRequest {

  my ( $sFirstArg ) = @_;

  my $sMyName = (caller 0)[3];

  #----------------------------------------------
  # Check whether the string is defined or not.
  #----------------------------------------------
  if ( defined $sFirstArg ) {
    if ( $sFirstArg eq "--versions" ) {
      Version::PrintAllVersions();
      #-----------------------------------------------------------
      # To indicate to the caller that the version request 
      # was accepted. The caller should probably normally exit...
      #-----------------------------------------------------------
      return 1; 
    } else {
      #----------------------------------------------------------
      # To indicate to the caller that no version listing was 
      # requested...
      #----------------------------------------------------------
      return 0; 
    }
  }
}


=pod

----------------------------------------------------------------
Function: void PrintAllVersions( void ) 

Description: 
  This prints out all the information about modules 
  which have called 'TellVersion'

Parameters:
  void:  no parameters are needed.

Return value:
  void:  no value is returned to the caller.
----------------------------------------------------------------

=cut

sub PrintAllVersions {
  #------------------------------------
  # Spin the hash and print stuff out.
  #------------------------------------
  my $sMyName = (caller 0)[3];
  
  print "Version::PrintAllVersions (begin)\n";

  foreach my $sTmpFileName (sort keys %lsFileVersionHash) {
    print "Version file:[$sTmpFileName] is [$lsFileVersionHash{$sTmpFileName}]\n";
  }

  print "Version::PrintAllVersions (end)\n";

  return;
}


1; 

